/*
   This library was downloaded from: http://www.mike95.com

   This library is copyright.  It may freely be used for personal purposes 
   if the restriction listed below is adhered to.
       Author: Michael Olivero
       Email:  mike95@mike95.com

   //===============================
   //Start of Restriction Definition
   //===============================
   Anyone can have full use of the library provided they keep this complete comment
   with the source.  Also I would like to ask if any changes are made to the
   code for efficiency reasons, please let me know so I may look into your change and
   likewise incorporate it into library.  If a suggestion makes it into the library,
   your credits will be added to this information.

   Authors of Computer related books are welcome to include this source code as part
   of their publishing, provided credit the Author and make note of where the source
   code was obtained from: http://www.mike95.com
   //=============================
   //End of Restriction Definition
   //=============================

   Description:
   Visit http://www.mike95.com/c_plusplus/classes/JStack/

   Standard LIFO (Last In First Out) datastructure.  It's public member functions
   are identical to the Java Stack public member functions (except for any Java specific
   Java related functions).
*/

#pragma once
#include <stdlib.h>

template <class Etype>
class JStack
{
public:
	JStack();	// Constructor
	virtual ~JStack();

	//Inspectors
	//==========
	const Etype & peek() const;
	int isEmpty() const { return m_pTOS == NULL; }
	int search( const Etype &item ) const;

	//Modifiers
	//=========
	void empty();
	Etype pop();
	void push( const Etype &item );

protected:
	class node
	{
	public:
		node( const Etype &val, node *ptr ) : value( val ), next( ptr ) {}
		Etype value;
		node *next;

	};

private:
	node* m_pTOS;
};


//===============================================================
//Implementation of constructor, destructor, and member functions
//Necessary location for appropriate template enstantiation.
//===============================================================
template <class Etype>
JStack<Etype>::JStack()
{
	m_pTOS = NULL;
}

template <class Etype>
JStack<Etype>::~JStack()
{
	empty();
}

template <class Etype>
const Etype &
JStack<Etype>::peek() const
{
	if ( m_pTOS == NULL )
		throw "JStack is Empty";

	return m_pTOS->value;
}

template <class Etype>
Etype
JStack<Etype>::pop()
{
	if ( m_pTOS == NULL )
		throw "JStack is Empty";

	node* old = m_pTOS;
	m_pTOS = m_pTOS->next;

	const Etype temp = old->value;
	delete old;

	return temp;
}

template <class Etype>
void
JStack<Etype>::empty()
{
	while ( m_pTOS != NULL )
	{
		node* old = m_pTOS;
		m_pTOS = m_pTOS->next;
		delete old;
	}
}

template <class Etype>
void
JStack<Etype>::push( const Etype &item )
{
	m_pTOS = new node( item, m_pTOS );
}

template <class Etype>
int
JStack<Etype>::search( const Etype &item ) const 
{
	const node* pJStack = m_pTOS;
	for( int count = 1; pJStack != NULL; count++ )
	{
		if ( pJStack->value == item )
			return count;
		pJStack = pJStack->next;
	}

	return -1;
}

